@@ -42,13 +42,6 @@ import click_completion
 # logging.basicConfig(level=logging.DEBUG) # level=10
 LOG = logging.getLogger("APRSD")
 
-LOG_LEVELS = {
-    "CRITICAL": logging.CRITICAL,
-    "ERROR": logging.ERROR,
-    "WARNING": logging.WARNING,
-    "INFO": logging.INFO,
-    "DEBUG": logging.DEBUG,
-}
 
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
 
@@ -175,10 +168,10 @@ def signal_handler(sig, frame):
 # to disable logging to stdout, but still log to file
 # use the --quiet option on the cmdln
 def setup_logging(config, loglevel, quiet):
-    log_level = LOG_LEVELS[loglevel]
+    log_level = utils.LOG_LEVELS[loglevel]
     LOG.setLevel(log_level)
-    log_format = "[%(asctime)s] [%(threadName)-12s] [%(levelname)-5.5s]" " %(message)s"
-    date_format = "%m/%d/%Y %I:%M:%S %p"
+    log_format = config["aprsd"].get("logformat", utils.DEFAULT_LOG_FORMAT)
+    date_format = config["aprsd"].get("dateformat", utils.DEFAULT_DATE_FORMAT)
     log_formatter = logging.Formatter(fmt=log_format, datefmt=date_format)
     log_file = config["aprsd"].get("logfile", None)
     if log_file:
@@ -206,6 +199,43 @@ def setup_logging(config, loglevel, quiet):
             imap_logger.addHandler(sh)
 
 
+@main.command()
+@click.option(
+    "--loglevel",
+    default="INFO",
+    show_default=True,
+    type=click.Choice(
+        ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG"],
+        case_sensitive=False,
+    ),
+    show_choices=True,
+    help="The log level to use for aprsd.log",
+)
+@click.option(
+    "-c",
+    "--config",
+    "config_file",
+    show_default=True,
+    default=utils.DEFAULT_CONFIG_FILE,
+    help="The aprsd config file to use for options.",
+)
+def check_version(loglevel, config_file):
+    config = utils.parse_config(config_file)
+
+    # Force setting the config to the modules that need it
+    # TODO(Walt): convert these modules to classes that can
+    # Accept the config as a constructor param, instead of this
+    # hacky global setting
+    email.CONFIG = config
+
+    setup_logging(config, loglevel, False)
+    level, msg = utils._check_version()
+    if level:
+        LOG.warning(msg)
+    else:
+        LOG.info(msg)
+
+
 @main.command()
 def sample_config():
     """This dumps the config to stdout."""
@@ -415,6 +445,7 @@ def server(
 
     if not quiet:
         click.echo("Load config")
+
     config = utils.parse_config(config_file)
 
     # Force setting the config to the modules that need it
@@ -424,9 +455,23 @@ def server(
     email.CONFIG = config
 
     setup_logging(config, loglevel, quiet)
+    level, msg = utils._check_version()
+    if level:
+        LOG.warning(msg)
+    else:
+        LOG.info(msg)
+    LOG.info("APRSD Started version: {}".format(aprsd.__version__))
+
+    flat_config = utils.flatten_dict(config)
+    LOG.info("Using CONFIG values:")
+    for x in flat_config:
+        if "password" in x or "aprsd.web.users.admin" in x:
+            LOG.info("{} = XXXXXXXXXXXXXXXXXXX".format(x))
+        else:
+            LOG.info("{} = {}".format(x, flat_config[x]))
+
     if config["aprsd"].get("trace", False):
         trace.setup_tracing(["method", "api"])
-    LOG.info("APRSD Started version: {}".format(aprsd.__version__))
     stats.APRSDStats(config)
 
     email_enabled = config["aprsd"]["email"].get("enabled", False)
@@ -459,12 +504,26 @@ def server(
         LOG.debug("Loading saved MsgTrack object.")
         messaging.MsgTrack().load()
 
+    rx_notify_queue = queue.Queue(maxsize=20)
     rx_msg_queue = queue.Queue(maxsize=20)
     tx_msg_queue = queue.Queue(maxsize=20)
-    msg_queues = {"rx": rx_msg_queue, "tx": tx_msg_queue}
+    msg_queues = {
+        "rx": rx_msg_queue,
+        "tx": tx_msg_queue,
+        "notify": rx_notify_queue,
+    }
 
     rx_thread = threads.APRSDRXThread(msg_queues=msg_queues, config=config)
     tx_thread = threads.APRSDTXThread(msg_queues=msg_queues, config=config)
+    if "watch_list" in config["aprsd"] and config["aprsd"]["watch_list"].get(
+        "enabled",
+        True,
+    ):
+        notify_thread = threads.APRSDNotifyThread(
+            msg_queues=msg_queues,
+            config=config,
+        )
+        notify_thread.start()
 
     if email_enabled:
         email_thread = email.APRSDEmailThread(msg_queues=msg_queues, config=config)
@@ -485,7 +544,7 @@ def server(
 
     if web_enabled:
         flask_enabled = True
-        app = flask.init_flask(config)
+        app = flask.init_flask(config, loglevel, quiet)
         app.run(
             host=config["aprsd"]["web"]["host"],
             port=config["aprsd"]["web"]["port"],
