@@ -9,7 +9,7 @@ import re
 import threading
 import time
 
-from aprsd import client, threads, utils
+from aprsd import client, stats, threads, trace, utils
 
 LOG = logging.getLogger("APRSD")
 
@@ -49,26 +49,29 @@ class MsgTrack:
         if cls._instance is None:
             cls._instance = super().__new__(cls)
             cls._instance.track = {}
-            cls._start_time = datetime.datetime.now()
+            cls._instance._start_time = datetime.datetime.now()
             cls._instance.lock = threading.Lock()
         return cls._instance
 
-    def add(self, msg):
+    def __getitem__(self, name):
         with self.lock:
-            key = int(msg.id)
-            self.track[key] = msg
-            self.total_messages_tracked += 1
+            return self.track[name]
 
-    def get(self, id):
+    def __iter__(self):
         with self.lock:
-            if id in self.track:
-                return self.track[id]
+            return iter(self.track)
 
-    def remove(self, id):
+    def keys(self):
         with self.lock:
-            key = int(id)
-            if key in self.track.keys():
-                del self.track[key]
+            return self.track.keys()
+
+    def items(self):
+        with self.lock:
+            return self.track.items()
+
+    def values(self):
+        with self.lock:
+            return self.track.values()
 
     def __len__(self):
         with self.lock:
@@ -82,12 +85,36 @@ class MsgTrack:
             result += "}"
             return result
 
+    def add(self, msg):
+        with self.lock:
+            key = int(msg.id)
+            self.track[key] = msg
+            stats.APRSDStats().msgs_tracked_inc()
+            self.total_messages_tracked += 1
+
+    def get(self, id):
+        with self.lock:
+            if id in self.track:
+                return self.track[id]
+
+    def remove(self, id):
+        with self.lock:
+            key = int(id)
+            if key in self.track.keys():
+                del self.track[key]
+
     def save(self):
-        """Save this shit to disk?"""
+        """Save any queued to disk?"""
+        LOG.debug("Save tracker to disk? {}".format(len(self)))
         if len(self) > 0:
             LOG.info("Saving {} tracking messages to disk".format(len(self)))
             pickle.dump(self.dump(), open(utils.DEFAULT_SAVE_FILE, "wb+"))
         else:
+            LOG.debug(
+                "Nothing to save, flushing old save file '{}'".format(
+                    utils.DEFAULT_SAVE_FILE,
+                ),
+            )
             self.flush()
 
     def dump(self):
@@ -159,11 +186,12 @@ class MessageCounter:
 
     _instance = None
     max_count = 9999
+    lock = None
 
     def __new__(cls, *args, **kwargs):
         """Make this a singleton class."""
         if cls._instance is None:
-            cls._instance = super().__new__(cls)
+            cls._instance = super().__new__(cls, *args, **kwargs)
             cls._instance.val = RawValue("i", 1)
             cls._instance.lock = threading.Lock()
         return cls._instance
@@ -196,7 +224,7 @@ class Message(metaclass=abc.ABCMeta):
     id = 0
 
     retry_count = 3
-    last_send_time = None
+    last_send_time = 0
     last_send_attempt = 0
 
     def __init__(self, fromcall, tocall, msg_id=None):
@@ -228,8 +256,20 @@ class RawMessage(Message):
         super().__init__(None, None, msg_id=None)
         self.message = message
 
-    def __repr__(self):
-        return self.message
+    def dict(self):
+        now = datetime.datetime.now()
+        last_send_age = None
+        if self.last_send_time:
+            last_send_age = str(now - self.last_send_time)
+        return {
+            "type": "raw",
+            "message": self.message.rstrip("\n"),
+            "raw": self.message.rstrip("\n"),
+            "retry_count": self.retry_count,
+            "last_send_attempt": self.last_send_attempt,
+            "last_send_time": str(self.last_send_time),
+            "last_send_age": last_send_age,
+        }
 
     def __str__(self):
         return self.message
@@ -245,12 +285,13 @@ class RawMessage(Message):
         cl = client.get_client()
         log_message(
             "Sending Message Direct",
-            repr(self).rstrip("\n"),
+            str(self).rstrip("\n"),
             self.message,
             tocall=self.tocall,
             fromcall=self.fromcall,
         )
-        cl.sendall(repr(self))
+        cl.sendall(str(self))
+        stats.APRSDStats().msgs_sent_inc()
 
 
 class TextMessage(Message):
@@ -265,28 +306,35 @@ class TextMessage(Message):
         # an ack?  Some messages we don't want to do this ever.
         self.allow_delay = allow_delay
 
-    def __repr__(self):
+    def dict(self):
+        now = datetime.datetime.now()
+
+        last_send_age = None
+        if self.last_send_time:
+            last_send_age = str(now - self.last_send_time)
+
+        return {
+            "id": self.id,
+            "type": "text-message",
+            "fromcall": self.fromcall,
+            "tocall": self.tocall,
+            "message": self.message.rstrip("\n"),
+            "raw": str(self).rstrip("\n"),
+            "retry_count": self.retry_count,
+            "last_send_attempt": self.last_send_attempt,
+            "last_send_time": str(self.last_send_time),
+            "last_send_age": last_send_age,
+        }
+
+    def __str__(self):
         """Build raw string to send over the air."""
-        return "{}>APRS::{}:{}{{{}\n".format(
+        return "{}>APZ100::{}:{}{{{}\n".format(
             self.fromcall,
             self.tocall.ljust(9),
             self._filter_for_send(),
             str(self.id),
         )
 
-    def __str__(self):
-        delta = "Never"
-        if self.last_send_time:
-            now = datetime.datetime.now()
-            delta = now - self.last_send_time
-        return "{}>{} Msg({})({}): '{}'".format(
-            self.fromcall,
-            self.tocall,
-            self.id,
-            delta,
-            self.message,
-        )
-
     def _filter_for_send(self):
         """Filter and format message string for FCC."""
         # max?  ftm400 displays 64, raw msg shows 74
@@ -309,12 +357,13 @@ class TextMessage(Message):
         cl = client.get_client()
         log_message(
             "Sending Message Direct",
-            repr(self).rstrip("\n"),
+            str(self).rstrip("\n"),
             self.message,
             tocall=self.tocall,
             fromcall=self.fromcall,
         )
-        cl.sendall(repr(self))
+        cl.sendall(str(self))
+        stats.APRSDStats().msgs_tx_inc()
 
 
 class SendMessageThread(threads.APRSDThread):
@@ -367,13 +416,14 @@ class SendMessageThread(threads.APRSDThread):
                 # tracking the time.
                 log_message(
                     "Sending Message",
-                    repr(msg).rstrip("\n"),
+                    str(msg).rstrip("\n"),
                     msg.message,
                     tocall=self.msg.tocall,
                     retry_number=msg.last_send_attempt,
                     msg_num=msg.id,
                 )
-                cl.sendall(repr(msg))
+                cl.sendall(str(msg))
+                stats.APRSDStats().msgs_tx_inc()
                 msg.last_send_time = datetime.datetime.now()
                 msg.last_send_attempt += 1
 
@@ -388,34 +438,30 @@ class AckMessage(Message):
     def __init__(self, fromcall, tocall, msg_id):
         super().__init__(fromcall, tocall, msg_id=msg_id)
 
-    def __repr__(self):
-        return "{}>APRS::{}:ack{}\n".format(
+    def dict(self):
+        now = datetime.datetime.now()
+        last_send_age = None
+        if self.last_send_time:
+            last_send_age = str(now - self.last_send_time)
+        return {
+            "id": self.id,
+            "type": "ack",
+            "fromcall": self.fromcall,
+            "tocall": self.tocall,
+            "raw": str(self).rstrip("\n"),
+            "retry_count": self.retry_count,
+            "last_send_attempt": self.last_send_attempt,
+            "last_send_time": str(self.last_send_time),
+            "last_send_age": last_send_age,
+        }
+
+    def __str__(self):
+        return "{}>APZ100::{}:ack{}\n".format(
             self.fromcall,
             self.tocall.ljust(9),
             self.id,
         )
 
-    def __str__(self):
-        return "From({}) TO({}) Ack ({})".format(self.fromcall, self.tocall, self.id)
-
-    def send_thread(self):
-        """Separate thread to send acks with retries."""
-        cl = client.get_client()
-        for i in range(self.retry_count, 0, -1):
-            log_message(
-                "Sending ack",
-                repr(self).rstrip("\n"),
-                None,
-                ack=self.id,
-                tocall=self.tocall,
-                retry_number=i,
-            )
-            cl.sendall(repr(self))
-            # aprs duplicate detection is 30 secs?
-            # (21 only sends first, 28 skips middle)
-            time.sleep(31)
-        # end_send_ack_thread
-
     def send(self):
         LOG.debug("Send ACK({}:{}) to radio.".format(self.tocall, self.id))
         thread = SendAckThread(self)
@@ -428,13 +474,13 @@ class AckMessage(Message):
         cl = client.get_client()
         log_message(
             "Sending ack",
-            repr(self).rstrip("\n"),
+            str(self).rstrip("\n"),
             None,
             ack=self.id,
             tocall=self.tocall,
             fromcall=self.fromcall,
         )
-        cl.sendall(repr(self))
+        cl.sendall(str(self))
 
 
 class SendAckThread(threads.APRSDThread):
@@ -442,8 +488,10 @@ class SendAckThread(threads.APRSDThread):
         self.ack = ack
         super().__init__("SendAck-{}".format(self.ack.id))
 
+    @trace.trace
     def loop(self):
         """Separate thread to send acks with retries."""
+        LOG.debug("SendAckThread loop start")
         send_now = False
         if self.ack.last_send_attempt == self.ack.retry_count:
             # we reached the send limit, don't send again
@@ -471,16 +519,18 @@ class SendAckThread(threads.APRSDThread):
             cl = client.get_client()
             log_message(
                 "Sending ack",
-                repr(self.ack).rstrip("\n"),
+                str(self.ack).rstrip("\n"),
                 None,
                 ack=self.ack.id,
                 tocall=self.ack.tocall,
                 retry_number=self.ack.last_send_attempt,
             )
-            cl.sendall(repr(self.ack))
+            cl.sendall(str(self.ack))
+            stats.APRSDStats().ack_tx_inc()
             self.ack.last_send_attempt += 1
             self.ack.last_send_time = datetime.datetime.now()
         time.sleep(5)
+        return True
 
 
 def log_packet(packet):
