@@ -3,7 +3,7 @@ import logging
 import threading
 
 import aprsd
-from aprsd import utils
+from aprsd import packets, plugin, utils
 
 LOG = logging.getLogger("APRSD")
 
@@ -181,6 +181,21 @@ class APRSDStats:
         else:
             last_aprsis_keepalive = "never"
 
+        pm = plugin.PluginManager()
+        plugins = pm.get_msg_plugins()
+        plugin_stats = {}
+
+        def full_name_with_qualname(obj):
+            return "{}.{}".format(
+                obj.__class__.__module__,
+                obj.__class__.__qualname__,
+            )
+
+        for p in plugins:
+            plugin_stats[full_name_with_qualname(p)] = p.message_count
+
+        wl = packets.WatchList()
+
         stats = {
             "aprsd": {
                 "version": aprsd.__version__,
@@ -189,6 +204,7 @@ class APRSDStats:
                 "memory_current_str": utils.human_size(self.memory),
                 "memory_peak": self.memory_peak,
                 "memory_peak_str": utils.human_size(self.memory_peak),
+                "watch_list": wl.callsigns,
             },
             "aprs-is": {
                 "server": self.aprsis_server,
@@ -209,6 +225,7 @@ class APRSDStats:
                 "recieved": self._email_rx,
                 "thread_last_update": last_update,
             },
+            "plugins": plugin_stats,
         }
         return stats
 
