@@ -6,7 +6,7 @@ import threading
 import time
 import tracemalloc
 
-from aprsd import client, messaging, plugin, stats, trace
+from aprsd import client, messaging, packets, plugin, stats, utils
 import aprslib
 
 LOG = logging.getLogger("APRSD")
@@ -67,40 +67,98 @@ class APRSDThread(threading.Thread, metaclass=abc.ABCMeta):
 
 class KeepAliveThread(APRSDThread):
     cntr = 0
+    checker_time = datetime.datetime.now()
 
     def __init__(self):
-        super().__init__("KeepAlive")
         tracemalloc.start()
+        super().__init__("KeepAlive")
 
     def loop(self):
         if self.cntr % 6 == 0:
             tracker = messaging.MsgTrack()
             stats_obj = stats.APRSDStats()
+            packets_list = packets.PacketList().packet_list
             now = datetime.datetime.now()
-            last_email = stats.APRSDStats().email_thread_time
+            last_email = stats_obj.email_thread_time
             if last_email:
-                email_thread_time = str(now - last_email)
+                email_thread_time = utils.strfdelta(now - last_email)
             else:
                 email_thread_time = "N/A"
 
+            last_msg_time = utils.strfdelta(now - stats_obj.aprsis_keepalive)
+
             current, peak = tracemalloc.get_traced_memory()
-            LOG.debug(
-                "Uptime ({}) Tracker({}) "
-                "Msgs: TX:{} RX:{} EmailThread: {} RAM: Current:{} Peak:{}".format(
-                    stats_obj.uptime,
-                    len(tracker),
-                    stats_obj.msgs_tx,
-                    stats_obj.msgs_rx,
-                    email_thread_time,
-                    current,
-                    peak,
-                ),
+            stats_obj.set_memory(current)
+            stats_obj.set_memory_peak(peak)
+            keepalive = "Uptime {} Tracker {} " "Msgs TX:{} RX:{} Last:{} Email:{} Packets:{} RAM Current:{} Peak:{}".format(
+                utils.strfdelta(stats_obj.uptime),
+                len(tracker),
+                stats_obj.msgs_tx,
+                stats_obj.msgs_rx,
+                last_msg_time,
+                email_thread_time,
+                len(packets_list),
+                utils.human_size(current),
+                utils.human_size(peak),
             )
+            LOG.debug(keepalive)
+            # Check version every hour
+            delta = now - self.checker_time
+            if delta > datetime.timedelta(hours=1):
+                self.checker_time = now
+                level, msg = utils._check_version()
+                if level:
+                    LOG.warning(msg)
         self.cntr += 1
         time.sleep(10)
         return True
 
 
+class APRSDNotifyThread(APRSDThread):
+    last_seen = {}
+
+    def __init__(self, msg_queues, config):
+        super().__init__("NOTIFY_MSG")
+        self.msg_queues = msg_queues
+        self.config = config
+        packets.WatchList(config=config)
+
+    def loop(self):
+        try:
+            packet = self.msg_queues["notify"].get(timeout=5)
+            wl = packets.WatchList()
+            if wl.callsign_in_watchlist(packet["from"]):
+                # NOW WE RUN through the notify plugins.
+                # If they return a msg, then we queue it for sending.
+                pm = plugin.PluginManager()
+                results = pm.notify(packet)
+                for reply in results:
+                    if reply is not messaging.NULL_MESSAGE:
+                        watch_list_conf = self.config["aprsd"]["watch_list"]
+
+                        msg = messaging.TextMessage(
+                            self.config["aprs"]["login"],
+                            watch_list_conf["alert_callsign"],
+                            reply,
+                        )
+                        self.msg_queues["tx"].put(msg)
+
+                wl.update_seen(packet)
+            else:
+                LOG.debug(
+                    "Ignoring packet from '{}'. Not in watch list.".format(
+                        packet["from"],
+                    ),
+                )
+
+            # Allows stats object to have latest info from the last_seen dict
+            LOG.debug("Packet processing complete")
+        except queue.Empty:
+            pass
+        # Continue to loop
+        return True
+
+
 class APRSDRXThread(APRSDThread):
     def __init__(self, msg_queues, config):
         super().__init__("RX_MSG")
@@ -114,6 +172,23 @@ class APRSDRXThread(APRSDThread):
     def loop(self):
         aprs_client = client.get_client()
 
+        # if we have a watch list enabled, we need to add filtering
+        # to enable seeing packets from the watch list.
+        if "watch_list" in self.config["aprsd"] and self.config["aprsd"][
+            "watch_list"
+        ].get("enabled", False):
+            # watch list is enabled
+            watch_list = self.config["aprsd"]["watch_list"].get(
+                "callsigns",
+                [],
+            )
+            # make sure the timeout is set or this doesn't work
+            if watch_list:
+                filter_str = "p/{}".format("/".join(watch_list))
+                aprs_client.set_filter(filter_str)
+            else:
+                LOG.warning("Watch list enabled, but no callsigns set.")
+
         # setup the consumer of messages and block until a messages
         try:
             # This will register a packet consumer with aprslib
@@ -176,7 +251,7 @@ class APRSDRXThread(APRSDThread):
         # Get singleton of the PM
         pm = plugin.PluginManager()
         try:
-            results = pm.run(fromcall=fromcall, message=message, ack=msg_id)
+            results = pm.run(packet)
             for reply in results:
                 found_command = True
                 # A plugin can return a null message flag which signals
@@ -195,7 +270,7 @@ class APRSDRXThread(APRSDThread):
                     LOG.debug("Got NULL MESSAGE from plugin")
 
             if not found_command:
-                plugins = pm.get_plugins()
+                plugins = pm.get_msg_plugins()
                 names = [x.command_name for x in plugins]
                 names.sort()
 
@@ -222,34 +297,50 @@ class APRSDRXThread(APRSDThread):
             msg_id=msg_id,
         )
         self.msg_queues["tx"].put(ack)
-        LOG.debug("Packet processing complete")
 
-    @trace.trace
     def process_packet(self, packet):
         """Process a packet recieved from aprs-is server."""
 
         try:
-            stats.APRSDStats().msgs_rx_inc()
+            LOG.debug("Adding packet to notify queue {}".format(packet["raw"]))
+            self.msg_queues["notify"].put(packet)
+            packets.PacketList().add(packet)
 
-            msg = packet.get("message_text", None)
-            msg_format = packet.get("format", None)
-            msg_response = packet.get("response", None)
-            if msg_format == "message" and msg:
-                # we want to send the message through the
-                # plugins
-                self.process_message_packet(packet)
-                return
-            elif msg_response == "ack":
-                self.process_ack_packet(packet)
-                return
+            # since we can see packets from anyone now with the
+            # watch list, we need to filter messages directly only to us.
+            tocall = packet.get("addresse", None)
+            if tocall == self.config["aprs"]["login"]:
+                stats.APRSDStats().msgs_rx_inc()
+                packets.PacketList().add(packet)
 
-            if msg_format == "mic-e":
-                # process a mic-e packet
-                self.process_mic_e_packet(packet)
-                return
+                msg = packet.get("message_text", None)
+                msg_format = packet.get("format", None)
+                msg_response = packet.get("response", None)
+                if msg_format == "message" and msg:
+                    # we want to send the message through the
+                    # plugins
+                    self.process_message_packet(packet)
+                    return
+                elif msg_response == "ack":
+                    self.process_ack_packet(packet)
+                    return
+
+                if msg_format == "mic-e":
+                    # process a mic-e packet
+                    self.process_mic_e_packet(packet)
+                    return
+            else:
+                LOG.debug(
+                    "Ignoring '{}' packet from '{}' to '{}'".format(
+                        packets.get_packet_type(packet),
+                        packet["from"],
+                        tocall,
+                    ),
+                )
 
         except (aprslib.ParseError, aprslib.UnknownFormat) as exp:
             LOG.exception("Failed to parse packet from aprs-is", exp)
+        LOG.debug("Packet processing complete")
 
 
 class APRSDTXThread(APRSDThread):
@@ -260,7 +351,8 @@ class APRSDTXThread(APRSDThread):
 
     def loop(self):
         try:
-            msg = self.msg_queues["tx"].get(timeout=0.1)
+            msg = self.msg_queues["tx"].get(timeout=5)
+            packets.PacketList().add(msg.dict())
             msg.send()
         except queue.Empty:
             pass
